import argparse
import asyncio
import logging.config

from zope.dottedname.resolve import resolve


@asyncio.coroutine
def start_logging():
    logging.config.fileConfig('logging.conf')


@asyncio.coroutine
def runner(argv, app=None, configfile=None,
           signals=None, config_string=None):
    loop = asyncio.get_event_loop()

    if not app:
        app = resolve("aio.app")

    parser = argparse.ArgumentParser(
        prog="aio",
        description='aio app runner')

    parser.add_argument(
        "-c", nargs="?",
        help="configuration file")

    parsed = parser.parse_known_args(argv)

    if parsed[0].c:
        configfile = parsed[0].c

    import aio.config
    app.config = yield from aio.config.parse_config(
        config=configfile, config_string=config_string)

    commands = app.config['aio:commands']

    parser.add_argument(
        "command", choices=commands,
        help="command to run")
    parser.add_argument(
        'nargs',
        default=[],
        help=argparse.SUPPRESS,
        nargs="*")

    try:
        parsed_args = parser.parse_args(argv)
    except (SystemExit, IndexError):
        parser.print_help()
        loop.stop()
        return

    from aio import signals as _signals    

    yield from start_logging()

    app.signals = signals or _signals.Signals()

    app.modules = []
    try:
        _modules = app.config['aio']['modules']
        for m in _modules.strip('').split('\n'):
            app.modules.append(resolve(m))
    except KeyError:
        pass

    if parsed_args.command in commands:
        try:
            task = resolve(commands[parsed_args.command])
        except Exception as e:
            import traceback
            traceback.print_exc()
            loop.stop()

        yield from task(parsed_args.nargs)

    else:
        parser.print_help()
        loop.stop()



import os
import io
import asyncio
import functools
from configparser import ConfigParser

import aio.app
from aio.testing import aiotest, aiofuturetest
from aio.testing.contextmanagers import redirect_all
from aio.app.testing import AioAppTestCase
from aio.app.runner import runner
from aio.signals import Signals
from aio.core.exceptions import MissingConfiguration

test_dir = os.path.dirname(__file__)

SCHEDULER_CONFIG = """
[aio:commands]
run: aio.app.cmd.cmd_run

[schedule:test]
every: 2
func: aio.app.tests._test_scheduler
"""
    

class RunCommandSchedulersTestCase(AioAppTestCase):

    @aiofuturetest
    def test_run_schedulers(self):

        class Counter:
            hit_count = 0
        counter = Counter()
        
        @asyncio.coroutine
        def scheduler(name):
            counter.hit_count += 1

        aio.app.tests._test_scheduler = scheduler
        
        yield from runner(
            ['run'],
            config_string=SCHEDULER_CONFIG)

        @asyncio.coroutine
        def test_complete():
            self.assertTrue(counter.hit_count == 3)

        return test_complete
